@@ -129,6 +129,11 @@ class FachoXML:
         self.xpath_for = {}
         self.extensions = []
 
+    @classmethod
+    def from_string(cls, document: str, namespaces: dict() = []) -> 'FachoXML':
+        xml = LXMLBuilder.from_string(document)
+        return FachoXML(xml, nsmap=namespaces)
+
     def append_element(self, elem, new_elem):
         #elem = self.find_or_create_element(xpath, append=append)
         #self.builder.append(elem, new_elem)
@@ -252,6 +257,9 @@ class FachoXML:
     def get_element_text(self, xpath, format_=str):
         xpath = self.fragment_prefix + self._path_xpath_for(xpath)
         elem = self.builder.xpath(self.root, xpath)
+        if elem is None:
+            raise AttributeError('xpath %s invalid' % (xpath))
+
         text = self.builder.get_text(elem)
         return format_(text)
 
