@@ -14,6 +14,7 @@ from contextlib import contextmanager
 from .data.dian import codelist
 from . import form
 from collections import defaultdict
+from pathlib import Path
 
 AMBIENTE_PRUEBAS = codelist.TipoAmbiente.by_name('Pruebas')['code']
 AMBIENTE_PRODUCCION = codelist.TipoAmbiente.by_name('Producción')['code']
@@ -25,8 +26,9 @@ SCHEME_AGENCY_ATTRS = {
 }
 
 
+pwd = Path(__file__).parent
 # RESOLUCION 0001: pagina 516
-POLICY_ID = 'https://facturaelectronica.dian.gov.co/politicadefirma/v2/politicadefirmav2.pdf'
+POLICY_ID = 'file://'+str(pwd)+'/data/dian/politicadefirmav2.pdf'
 POLICY_NAME = u'Política de firma para facturas electrónicas de la República de Colombia.'
 
 
@@ -49,8 +51,7 @@ NAMESPACES = {
 }
 
 def fe_from_string(document: str) -> FachoXML:
-    xml = LXMLBuilder.from_string(document)
-    return FachoXML(xml, nsmap=NAMESPACES)
+    return FeXML.from_string(document)
 
 from contextlib import contextmanager
 @contextmanager
@@ -69,6 +70,7 @@ def mock_xades_policy():
         mock.return_value = UrllibPolicyMock()
         yield
 
+        
 class FeXML(FachoXML):
 
     def __init__(self, root, namespace):
@@ -79,6 +81,10 @@ class FeXML(FachoXML):
         self._cn = root.rstrip('/')
         #self.find_or_create_element(self._cn)
 
+    @classmethod
+    def from_string(cls, document: str) -> 'FeXML':
+        return super().from_string(document, namespaces=NAMESPACES)
+    
     def tostring(self, **kw):
         return super().tostring(**kw)\
             .replace("fe:", "")\
@@ -185,14 +191,14 @@ class DianXMLExtensionCUFE(DianXMLExtensionCUDFE):
             '%s' % build_vars['NumFac'],
             '%s' % build_vars['FecFac'],
             '%s' % build_vars['HoraFac'],
-            form.Amount(build_vars['ValorBruto']).format('%.02f'),
+            form.Amount(build_vars['ValorBruto']).truncate_as_string(2),
             CodImpuesto1,
-            form.Amount(build_vars['ValorImpuestoPara'].get(CodImpuesto1, 0.0)).format('%.02f'),
+            build_vars['ValorImpuestoPara'].get(CodImpuesto1, form.Amount(0.0)).truncate_as_string(2),
             CodImpuesto2,
-            form.Amount(build_vars['ValorImpuestoPara'].get(CodImpuesto2, 0.0)).format('%.02f'),
+            build_vars['ValorImpuestoPara'].get(CodImpuesto2, form.Amount(0.0)).truncate_as_string(2),
             CodImpuesto3,
-            form.Amount(build_vars['ValorImpuestoPara'].get(CodImpuesto3, 0.0)).format('%.02f'),
-            form.Amount(build_vars['ValorTotalPagar']).format('%.02f'),
+            build_vars['ValorImpuestoPara'].get(CodImpuesto3, form.Amount(0.0)).truncate_as_string(2),
+            build_vars['ValorTotalPagar'].truncate_as_string(2),
             '%s' % build_vars['NitOFE'],
             '%s' % build_vars['NumAdq'],
             '%s' % build_vars['ClTec'],
@@ -222,14 +228,14 @@ class DianXMLExtensionCUDE(DianXMLExtensionCUDFE):
             '%s' % build_vars['NumFac'],
             '%s' % build_vars['FecFac'],
             '%s' % build_vars['HoraFac'],
-            form.Amount(build_vars['ValorBruto']).format('%.02f'),
+            form.Amount(build_vars['ValorBruto']).truncate_as_string(2),
             CodImpuesto1,
-            form.Amount(build_vars['ValorImpuestoPara'].get(CodImpuesto1, 0.0)).format('%.02f'),
+            form.Amount(build_vars['ValorImpuestoPara'].get(CodImpuesto1, 0.0)).truncate_as_string(2),
             CodImpuesto2,
-            form.Amount(build_vars['ValorImpuestoPara'].get(CodImpuesto2, 0.0)).format('%.02f'),
+            form.Amount(build_vars['ValorImpuestoPara'].get(CodImpuesto2, 0.0)).truncate_as_string(2),
             CodImpuesto3,
-            form.Amount(build_vars['ValorImpuestoPara'].get(CodImpuesto3, 0.0)).format('%.02f'),
-            form.Amount(build_vars['ValorTotalPagar']).format('%.02f'),
+            form.Amount(build_vars['ValorImpuestoPara'].get(CodImpuesto3, 0.0)).truncate_as_string(2),
+            form.Amount(build_vars['ValorTotalPagar']).truncate_as_string(2),
             '%s' % build_vars['NitOFE'],
             '%s' % build_vars['NumAdq'],
             '%s' % build_vars['Software-PIN'],
@@ -277,16 +283,24 @@ class DianXMLExtensionSoftwareSecurityCode(FachoXMLExtension):
 class DianXMLExtensionSigner:
 
     def __init__(self, pkcs12_path, passphrase=None, mockpolicy=False):
-        self._pkcs12_path = pkcs12_path
+        self._pkcs12_data = open(pkcs12_path, 'rb').read()
         self._passphrase = None
         self._mockpolicy = mockpolicy
         if passphrase:
             self._passphrase = passphrase.encode('utf-8')
 
     @classmethod
-    def from_pkcs12(self, filepath, password=None):
-        p12 = OpenSSL.crypto.load_pkcs12(open(filepath, 'rb').read(), password)
-
+    def from_bytes(cls, data, passphrase=None, mockpolicy=False):
+        self = cls.__new__(cls)
+        
+        self._pkcs12_data = data
+        self._passphrase = None
+        self._mockpolicy = mockpolicy
+        if passphrase:
+            self._passphrase = passphrase.encode('utf-8')
+            
+        return self
+    
     def sign_xml_string(self, document):
         xml = LXMLBuilder.from_string(document)
         signature = self.sign_xml_element(xml)
@@ -341,7 +355,7 @@ class DianXMLExtensionSigner:
             POLICY_NAME,
             xmlsig.constants.TransformSha256)
         ctx = xades.XAdESContext(policy)
-        ctx.load_pkcs12(OpenSSL.crypto.load_pkcs12(open(self._pkcs12_path, 'rb').read(),
+        ctx.load_pkcs12(OpenSSL.crypto.load_pkcs12(self._pkcs12_data,
                                                    self._passphrase))
 
         if self._mockpolicy:
@@ -360,6 +374,7 @@ class DianXMLExtensionSigner:
         extcontent = fachoxml.builder.xpath(fachoxml.root, './ext:UBLExtensions/ext:UBLExtension[2]/ext:ExtensionContent')
         fachoxml.append_element(extcontent, signature)
 
+        
 class DianXMLExtensionAuthorizationProvider(FachoXMLExtension):
     # RESOLUCION 0004: pagina 176
 
@@ -429,7 +444,7 @@ class DianZIP:
     MAX_FILES = 50
 
     def __init__(self, file_like):
-        self.zipfile = zipfile.ZipFile(file_like, mode='w')
+        self.zipfile = zipfile.ZipFile(file_like, mode='w', compression=zipfile.ZIP_DEFLATED)
         self.num_files = 0
 
     def add_invoice_xml(self, name, xml_data):
@@ -452,8 +467,8 @@ class DianZIP:
 
 class DianXMLExtensionSignerVerifier:
 
-    def __init__(self, pkcs12_path, passphrase=None, mockpolicy=False):
-        self._pkcs12_path = pkcs12_path
+    def __init__(self, pkcs12_path_or_bytes, passphrase=None, mockpolicy=False):
+        self._pkcs12_path_or_bytes = pkcs12_path_or_bytes
         self._passphrase = None
         self._mockpolicy = mockpolicy
         if passphrase:
@@ -470,7 +485,12 @@ class DianXMLExtensionSignerVerifier:
         fachoxml.root.append(signature)
 
         ctx = xades.XAdESContext()
-        ctx.load_pkcs12(OpenSSL.crypto.load_pkcs12(open(self._pkcs12_path, 'rb').read(),
+
+        pkcs12_data = self._pkcs12_path_or_bytes
+        if isinstance(self._pkcs12_path_or_bytes, str):
+            pkcs12_data = open(self._pkcs12_path_or_bytes, 'rb').read()
+
+        ctx.load_pkcs12(OpenSSL.crypto.load_pkcs12(pkcs12_data,
                                                    self._passphrase))
 
         try:
