@@ -4,6 +4,7 @@
 import hashlib
 from functools import reduce
 import copy
+import dataclasses
 from dataclasses import dataclass
 from datetime import datetime, date
 from collections import defaultdict
@@ -53,8 +54,8 @@ class AmountCollection(Collection):
         return total
 
 class Amount:
-    def __init__(self, amount: int or float or Amount, currency: Currency = Currency('COP')):
-
+    def __init__(self, amount: int or float or str or Amount, currency: Currency = Currency('COP'), precision = DECIMAL_PRECISION):
+        self.precision = precision
         #DIAN 1.7.-2020: 1.2.3.1
         if isinstance(amount, Amount):
             if amount < Amount(0.0):
@@ -63,42 +64,60 @@ class Amount:
             self.amount = amount.amount
             self.currency = amount.currency
         else:
-            if amount < 0:
+            if float(amount) < 0:
                 raise ValueError('amount must be positive >= 0')
 
-            self.amount = Decimal(amount, decimal.Context(prec=DECIMAL_PRECISION,
+            self.amount = Decimal(amount, decimal.Context(prec=self.precision,
                                                           #DIAN 1.7.-2020: 1.2.1.1
                                                           rounding=decimal.ROUND_HALF_EVEN ))
             self.currency = currency
 
+    def fromNumber(self, val):
+        return Amount(val, currency=self.currency)
+    
+    def round(self, prec):
+        return Amount(round(self.amount, prec), currency=self.currency)
 
     def __round__(self, prec):
         return round(self.amount, prec)
 
     def __str__(self):
-        return '%.06f' % self.amount
+        return str(self.float())
 
     def __lt__(self, other):
         if not self.is_same_currency(other):
             raise AmountCurrencyError()
-        return round(self.amount, DECIMAL_PRECISION) < round(other, 2)
+        return round(self.amount, self.precision) < round(other, 2)
 
     def __eq__(self, other):
         if not self.is_same_currency(other):
             raise AmountCurrencyError()
-        return round(self.amount, DECIMAL_PRECISION) == round(other.amount, DECIMAL_PRECISION)
+        return round(self.amount, self.precision) == round(other.amount, self.precision)
 
-    def __add__(self, other):
+    def _cast(self, val):
+        if type(val) in [int, float]:
+            return self.fromNumber(val)
+        if isinstance(val, Amount):
+            return val
+        if isinstance(val, Decimal):
+            return self.fromNumber(float(val))
+
+        raise TypeError("cant cast %s to amount" % (type(val)))
+    
+    def __add__(self, rother):
+        other = self._cast(rother)
         if not self.is_same_currency(other):
             raise AmountCurrencyError()
         return Amount(self.amount + other.amount, self.currency)
 
-    def __sub__(self, other):
+    def __sub__(self, rother):
+        other = self._cast(rother)
         if not self.is_same_currency(other):
             raise AmountCurrencyError()
         return Amount(self.amount - other.amount, self.currency)
 
-    def __mul__(self, other):
+    def __mul__(self, rother):
+        other = self._cast(rother)
         if not self.is_same_currency(other):
             raise AmountCurrencyError()
         return Amount(self.amount * other.amount, self.currency)
@@ -106,8 +125,9 @@ class Amount:
     def is_same_currency(self, other):
         return self.currency == other.currency
 
-    def format(self, formatter):
-        return formatter % self.float()
+    def truncate_as_string(self, prec):
+        parts = str(self.float()).split('.', 1)
+        return '%s.%s' % (parts[0], parts[1][0:prec].ljust(prec,'0'))
 
     def float(self):
         return float(round(self.amount, DECIMAL_PRECISION))
@@ -116,27 +136,25 @@ class Amount:
 class Quantity:
     
     def __init__(self, val, code):
-        if not isinstance(val, int):
-            raise ValueError('val expected int')
+        if type(val) not in [float, int]:
+            raise ValueError('val expected int or float')
         if code not in codelist.UnidadesMedida:
             raise ValueError("code [%s] not found" % (code))
 
-        self.value = val
+        self.value = Amount(val)
         self.code = code
 
     def __mul__(self, other):
-        if isinstance(other, Amount):
-            return Amount(self.value) * other
         return self.value * other
 
     def __lt__(self, other):
-        if isinstance(other, Amount):
-            return Amount(self.value) < other
         return self.value < other
 
     def __str__(self):
         return str(self.value)
 
+    def __repr__(self):
+        return str(self)
 
 @dataclass
 class Item:
@@ -323,11 +341,15 @@ class Price:
     amount: Amount
     type_code: str
     type: str
+    quantity: int = 1
 
     def __post_init__(self):
         if self.type_code not in codelist.CodigoPrecioReferencia:
             raise ValueError("type_code [%s] not found" % (self.type_code))
+        if not isinstance(self.quantity, int):
+            raise ValueError("quantity must be int")
 
+        self.amount *= self.quantity
 
 @dataclass
 class PaymentMean:
@@ -378,6 +400,52 @@ class InvoiceDocumentReference(BillingReference):
     date: fecha de emision de la nota credito relacionada
     """
 
+@dataclass
+class AllowanceChargeReason:
+    code: str
+    reason: str
+
+    def __post_init__(self):
+        if self.code not in codelist.CodigoDescuento:
+            raise ValueError("code [%s] not found" % (self.code))
+
+
+@dataclass
+class AllowanceCharge:
+    #DIAN 1.7.-2020: FAQ03
+    charge_indicator: bool = True
+    amount: Amount = Amount(0.0)
+    reason: AllowanceChargeReason = None
+
+    #Valor Base para calcular el descuento o el cargo
+    base_amount: typing.Optional[Amount] = Amount(0.0)
+    
+    # Porcentaje: Porcentaje que aplicar.
+    multiplier_factor_numeric: Amount = Amount(1.0)
+    
+    def isCharge(self):
+        return self.charge_indicator == True
+
+    def isDiscount(self):
+        return self.charge_indicator == False
+
+    def asCharge(self):
+        self.charge_indicator = True
+
+    def asDiscount(self):
+        self.charge_indicator = False
+
+    def hasReason(self):
+        return self.reason is not None
+
+    def set_base_amount(self, amount):
+        self.base_amount = amount
+
+class AllowanceChargeAsDiscount(AllowanceCharge):
+    def __init__(self, amount: Amount = Amount(0.0)):
+        self.charge_indicator = False
+        self.amount = amount
+
 @dataclass
 class InvoiceLine:
     # RESOLUCION 0004: pagina 155
@@ -391,9 +459,31 @@ class InvoiceLine:
     # de subtotal
     tax: typing.Optional[TaxTotal]
 
+    allowance_charge: typing.List[AllowanceCharge] = dataclasses.field(default_factory=list)
+
+    def add_allowance_charge(self, charge):
+        if not isinstance(charge, AllowanceCharge):
+            raise TypeError('charge invalid type expected AllowanceCharge')
+        charge.set_base_amount(self.total_amount_without_charge)
+        self.allowance_charge.add(charge)
+
+    @property
+    def total_amount_without_charge(self):
+        return (self.quantity * self.price.amount)
+    
     @property
     def total_amount(self):
-        return self.quantity * self.price.amount
+        charge = AmountCollection(self.allowance_charge)\
+            .filter(lambda charge: charge.isCharge())\
+            .map(lambda charge: charge.amount)\
+            .sum()
+
+        discount = AmountCollection(self.allowance_charge)\
+            .filter(lambda charge: charge.isDiscount())\
+            .map(lambda charge: charge.amount)\
+            .sum()
+
+        return self.total_amount_without_charge + charge - discount
 
     @property
     def total_tax_inclusive_amount(self):
@@ -441,37 +531,6 @@ class LegalMonetaryTotal:
             - self.prepaid_amount
 
 
-@dataclass
-class AllowanceChargeReason:
-    code: str
-    reason: str
-
-    def __post_init__(self):
-        if self.code not in codelist.CodigoDescuento:
-            raise ValueError("code [%s] not found" % (self.code))
-
-
-@dataclass
-class AllowanceCharge:
-    #DIAN 1.7.-2020: FAQ03
-    charge_indicator: bool = True
-    amount: Amount = Amount(0.0)
-    reason: AllowanceChargeReason = None
-
-    def isCharge(self):
-        return self.charge_indicator == True
-
-    def isDiscount(self):
-        return self.charge_indicator == False
-
-    def asCharge(self):
-        self.charge_indicator = True
-
-    def asDiscount(self):
-        self.charge_indicator = False
-
-    def hasReason(self):
-        return self.reason is not None
 
 class NationalSalesInvoiceDocumentType(str):
     def __str__(self):
@@ -570,7 +629,7 @@ class Invoice:
 
         self.invoice_operation_type = operation
 
-    def add_allownace_charge(self, charge: AllowanceCharge):
+    def add_allowance_charge(self, charge: AllowanceCharge):
         self.invoice_allowance_charge.append(charge)
 
     def add_invoice_line(self, line: InvoiceLine):
@@ -618,11 +677,22 @@ class Invoice:
         #DIAN 1.7.-2020: FAU14
         self.invoice_legal_monetary_total.calculate()
 
+    def _refresh_charges_base_amount(self):
+        if self.invoice_allowance_charge:
+            for invline in self.invoice_lines:
+                if invline.allowance_charge:
+                    # TODO actualmente solo uno de los cargos es permitido
+                    raise ValueError('allowance charge in invoice exclude invoice line')
+            
+        # cargos a nivel de factura
+        for charge in self.invoice_allowance_charge:
+            charge.set_base_amount(self.invoice_legal_monetary_total.line_extension_amount)
+        
     def calculate(self):
         for invline in self.invoice_lines:
             invline.calculate()
         self._calculate_legal_monetary_total()
-
+        self._refresh_charges_base_amount()
 
 class NationalSalesInvoice(Invoice):
     def __init__(self):
