@@ -55,6 +55,13 @@ def test_invoicesimple_zip(simple_invoice):
     with fe.DianZIP(zipdata) as dianzip:
         name_invoice = dianzip.add_invoice_xml(simple_invoice.invoice_ident, str(xml_invoice))
 
+    # el zip ademas de archivar debe comprimir los archivos
+    # de lo contrario la DIAN lo rechaza
+    with zipfile.ZipFile(zipdata) as dianzip:
+        dianzip.testzip()
+        for zipinfo in dianzip.infolist():
+            assert zipinfo.compress_type == zipfile.ZIP_DEFLATED, "se espera el zip comprimido"
+
     with zipfile.ZipFile(zipdata) as dianzip:
         xml_data = dianzip.open(name_invoice).read().decode('utf-8')
         assert xml_data == str(xml_invoice)
@@ -112,7 +119,7 @@ def test_invoice_cufe(simple_invoice_without_lines):
     simple_invoice.invoice_supplier.ident = form.PartyIdentification('700085371', '5', '31')
     simple_invoice.invoice_customer.ident = form.PartyIdentification('800199436', '5', '31')
     simple_invoice.add_invoice_line(form.InvoiceLine(
-        quantity = form.Quantity(1, '94'),
+        quantity = form.Quantity(1.00, '94'),
         description = 'producto',
         item = form.StandardItem(111),
         price = form.Price(form.Amount(1_500_000), '01', ''),
@@ -170,6 +177,7 @@ def test_invoice_cufe(simple_invoice_without_lines):
     assert cufe == '8bb918b19ba22a694f1da11c643b5e9de39adf60311cf179179e9b33381030bcd4c3c3f156c506ed5908f9276f5bd9b4'
 
 
+
 def test_credit_note_cude(simple_credit_note_without_lines):
     simple_invoice = simple_credit_note_without_lines
     simple_invoice.invoice_ident = '8110007871'
